::*******************************************************************************
::
:: E M B E D D E D   W I Z A R D   P R O J E C T
::
::                                                Copyright (c) TARA Systems GmbH
::                                    written by Paul Banach and Manfred Schweyer
::
::
:: Embedded Wizard Post Process script for automatic updating of the MPLAB
:: project (color format, screen orientation, generated source files,
:: Embedded Wizard runtime environment libraries and source files.
:: Call this script from Embedded Wizard Studio. A Post Process can be set as
:: Profil parameter.
::
::*******************************************************************************

@echo off


::set EmWi_ColorFormat=RGB565
::set EmWi_SurfaceRotation=0
::set EmWi_OutputSrcFiles=Application.c Core.c DefaultClut.c Effects.c Graphics.c Resources.c Views.c

echo Embedded Wizard post processing (Embedded Wizard version: %EmWi_Version%)
echo project:            %EmWi_ProjectFile%
echo profile:            %EmWi_Profile%
echo platform:           %EmWi_Platform%
echo screen orientation: %EmWi_SurfaceRotation%
echo screen size:        %EmWi_ScreenSize%
echo.
echo EmWi_ColorFormat = %EmWi_ColorFormat%
echo EmWi_SurfaceRotation = %EmWi_SurfaceRotation%
echo EmWi_OutputSrcFiles = %EmWi_OutputSrcFiles%

SET mypath=%~dp0
SET realativepath=%mypath:~0,-1%

:: Call a PowerShell script to modify the MPLABX project configuration file
PowerShell.exe -ExecutionPolicy Bypass -Command "& '%realativepath%\MPLAB_ew_post_process_power_shell_script.ps1'"


if ERRORLEVEL == 10 (
 echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 echo This Embedded Wizard Post Process needs at least V4.0 of Windows Powershell
 echo Powershell update can be found at:
 echo https://docs.microsoft.com/de-de/powershell/scripting/setup/installing-windows-powershell
 echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 exit /B 1
)

if ERRORLEVEL == 8 (
 echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 echo The selected color format '%EmWi_ColorFormat%' is not supported!
 echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 exit /B 1
)

if ERRORLEVEL == 6 (
 echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 echo The MPLAB project was not found!
 echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 exit /B 1
)

if ERRORLEVEL == 1 (
 echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 echo Unknown error occured during Embedded Wizard Post Process!
 echo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 exit /B 1
)

:: Find the MPLABX install directory
:: Adapt this path to your environment
:: The latest version below this path will be selected
call :findMplab "C:\Program Files (x86)\Microchip\MPLABX"
echo Using MPLAB installation below %MPLABX_PATH_l%


SET MakefilesGenerator=%MPLABX_PATH%\mplab_platform\bin\prjMakefilesGenerator.bat

If Not Exist %MakefilesGenerator% (
  echo %MakefilesGenerator% not found!
  echo please modifiy the path to MPLABX in %realativepath%\MPLAB_ew_post_process.cmd
  exit /B 1
)

:: update the MPLABX project by calling prjMakefilesGenerator.bat
start "updating MPLABX project (running prjMakefilesGenerator.bat) ..." cmd /Q  /C "%MakefilesGenerator%" %realativepath%\..\firmware\EmbeddedWizard-PIC32MZ-DA-MEB-II.X

echo MPLABX project updated!
echo Please change to MPLABX window and rebuild the project.
exit /B 0


:: function findMplab
:findMplab
  set MPLABX_PATH_=%~dspf1
  set MPLABX_PATH_l_=%~dpf1
  cd /d %MPLABX_PATH_%

  for /f "tokens=*" %%1 in ('dir /b /ad /on v*') do (
    set latestdir=%%1
  )

  set MPLABX_PATH=%MPLABX_PATH_%\%latestdir%
  set MPLABX_PATH_l=%MPLABX_PATH_l_%\%latestdir%


