/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target.
*   This template is responsible to configurate the entire system (CPU clock,
*   memory, qspi, etc).
*
*******************************************************************************/
#include "system_config.h"
#include "system_definitions.h"
#include "ew_bsp_system.h"

#if EW_USE_EXTERNAL_FLASH == 1
#include "peripheral/sqi/plib_sqi.h"

// *****************************************************************************
// *****************************************************************************
// Section: Enumerations
// *****************************************************************************
// *****************************************************************************
typedef enum {

    APP_SQI_ID_0 = SQI_ID_0,
    APP_SQI_NUMBER_OF_MODULES

} APP_SQI_MODULE_ID;

typedef enum {

    APP_SQI_XFER_MODE_XIP = SQI_XFER_MODE_XIP,
    APP_SQI_XFER_MODE_DMA = SQI_XFER_MODE_DMA,
    APP_SQI_XFER_MODE_PIO = SQI_XFER_MODE_PIO

} APP_SQI_XFER_MODE;

typedef enum {

    APP_SQI_DATA_MODE_3 = SQI_DATA_MODE_3,
    APP_SQI_DATA_MODE_0 = SQI_DATA_MODE_0

} APP_SQI_DATA_MODE;

typedef enum {

    APP_SQI_DATA_FORMAT_LSBF = SQI_DATA_FORMAT_LSBF,
    APP_SQI_DATA_FORMAT_MSBF = SQI_DATA_FORMAT_MSBF

} APP_SQI_DATA_FORMAT;

typedef enum {

    APP_SQI_DATA_OEN_QUAD = SQI_DATA_OEN_QUAD,
    APP_SQI_DATA_OEN_DUAL = SQI_DATA_OEN_DUAL,
    APP_SQI_DATA_OEN_SINGLE = SQI_DATA_OEN_SINGLE

} APP_SQI_DATA_OEN;

typedef enum {

    APP_SQI_CS_OEN_BOTH = SQI_CS_OEN_BOTH,
    APP_SQI_CS_OEN_1 = SQI_CS_OEN_1,
    APP_SQI_CS_OEN_0 = SQI_CS_OEN_0,
    APP_SQI_CS_OEN_NONE = SQI_CS_OEN_NONE

} APP_SQI_CS_OEN;

typedef enum {

    APP_SQI_CMD_RECEIVE = SQI_CMD_RECEIVE,
    APP_SQI_CMD_TRANSMIT = SQI_CMD_TRANSMIT,
    APP_SQI_CMD_IDLE = SQI_CMD_IDLE

} APP_SQI_XFER_CMD;

typedef enum {

    APP_SQI_LANE_QUAD = SQI_LANE_QUAD,
    APP_SQI_LANE_DUAL = SQI_LANE_DUAL,
    APP_SQI_LANE_SINGLE = SQI_LANE_SINGLE

} APP_SQI_LANE_MODE;

typedef enum {

    APP_SQI_CS_1 = SQI_CS_1,
    APP_SQI_CS_0 = SQI_CS_0

} APP_SQI_CS_NUM;

typedef enum {

    APP_ADDR_BYTES_4 = ADDR_BYTES_4,
    APP_ADDR_BYTES_3 = ADDR_BYTES_3,
    APP_ADDR_BYTES_2 = ADDR_BYTES_2,
    APP_ADDR_BYTES_1 = ADDR_BYTES_1,
    APP_ADDR_BYTES_0 = ADDR_BYTES_0

} APP_SQI_ADDR_BYTES;

typedef enum {

    APP_DUMMY_BYTES_7 = DUMMY_BYTES_7,
    APP_DUMMY_BYTES_6 = DUMMY_BYTES_6,
    APP_DUMMY_BYTES_5 = DUMMY_BYTES_5,
    APP_DUMMY_BYTES_4 = DUMMY_BYTES_4,
    APP_DUMMY_BYTES_3 = DUMMY_BYTES_3,
    APP_DUMMY_BYTES_2 = DUMMY_BYTES_2,
    APP_DUMMY_BYTES_1 = DUMMY_BYTES_1,
    APP_DUMMY_BYTES_0 = DUMMY_BYTES_0

} APP_SQI_DUMMY_BYTES;

typedef enum {

    APP_MODE_BYTES_3 = MODE_BYTES_3,
    APP_MODE_BYTES_2 = MODE_BYTES_2,
    APP_MODE_BYTES_1 = MODE_BYTES_1,
    APP_MODE_BYTES_0 = MODE_BYTES_0

} APP_SQI_MODE_BYTES;

typedef enum {

    APP_SQI_DMAERROR = 11,
    APP_SQI_PKTCOMP = SQI_PKTCOMP,
    APP_SQI_BDDONE = SQI_BDDONE,
    APP_SQI_CONTHR = SQI_CONTHR,
    APP_SQI_CONEMPTY = SQI_CONEMPTY,
    APP_SQI_CONFULL = SQI_CONFULL,
    APP_SQI_RXTHR = SQI_RXTHR,
    APP_SQI_RXEMPTY = SQI_RXEMPTY,
    APP_SQI_RXFULL = SQI_RXFULL,
    APP_SQI_TXTHR = SQI_TXTHR,
    APP_SQI_TXEMPTY = SQI_TXEMPTY,
    APP_SQI_TXFULL = SQI_TXFULL

} APP_SQI_INTERRUPTS;

typedef enum {

    APP_BD_STATE_DISABLED = BD_STATE_DISABLED,
    APP_BD_STATE_DONE = BD_STATE_DONE,
    APP_BD_STATE_PROCESSING_DATA = BD_STATE_PROCESSING_DATA,
    APP_BD_STATE_BEING_FETCHED = BD_STATE_BEING_FETCHED,
    APP_BD_STATE_FETCH_REQ_PENDING = BD_STATE_FETCH_REQ_PENDING,
    APP_BD_STATE_IDLE = BD_STATE_IDLE

} APP_SQI_BD_STATE;

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************
// *****************************************************************************
#define MAX_WRITE_BUF_DEPTH                 8
#define MAX_READ_BUF_DEPTH                  8
#define XIP_UNCACHED_ADDR_MASK              0xF0000000
#define XIP_CACHED_ADDR_MASK                0xD0000000
#define SQI_NUM_BUFFER_DESC                 256

//Flash Commands
#define FLASH_NOP                           0x00
#define FLASH_RSTEN                         0x66
#define FLASH_RST                           0x99
#define FLASH_EQIO                          0x38
#define FLASH_WEN                           0x06
#define FLASH_UNPROTECT_GLOBAL              0x98
#define FLASH_QJID                          0xAF
#define FLASH_ERASE                         0xC7
#define FLASH_PAGE_WRITE                    0x02
#define FLASH_FAST_READ                     0x0B
#define FLASH_RDSR                          0x05
#define FLASH_BUSY                          0x01
#define FLASH_READY                         0x00
#define FLASH_BUSY_MASK                     0x08
#define FLASH_BUSY_MASK_DUPLICATE           0x01

#define FLASH_PAGE_SIZE                     256
//Flash device ID
#define FLASH_JEDECID_B                     0x4226BF


typedef enum
{
    /* An error has occurred.*/
    SQI_STATUS_FAILURE = 0,

    /* No errors occurred*/
    SQI_STATUS_SUCCESS = 1,

} SQI_STATUS;


uint32_t __attribute__((coherent)) jedecID;


void APP_SQI_Initialize(void)
{
  // Set up SQI Configuration (SQI1CFG) Register
  PLIB_SQI_ConfigWordSet(
    APP_SQI_ID_0,
    1,
    APP_SQI_CS_OEN_1,
    APP_SQI_DATA_OEN_QUAD,
    0, // Resets control, transmit, receive buffers and state machines
    1, // Burst Enable (always set to '1')
    0, // SQID2 doesn?t act as HOLD# signal in single and dual lane modes
          0, // SQID3 doesn?t act as WP# signal in single and dual lane modes
    0, // Receive latch is not active in transmit mode
          APP_SQI_DATA_FORMAT_MSBF,
    APP_SQI_DATA_MODE_0,
    APP_SQI_XFER_MODE_PIO );

  // Configure SQI1CLKCON
  PLIB_SQI_ClockDividerSet(APP_SQI_ID_0, CLK_DIV_1);
  PLIB_SQI_ClockEnable(APP_SQI_ID_0);
  while(!PLIB_SQI_ClockIsStable(APP_SQI_ID_0));
}


inline void _APP_SQI_BufferThrSet(uint16_t conThr, uint16_t txThr, uint16_t rxThr)
{
  if (conThr != 0)
    PLIB_SQI_ControlBufferThresholdSet(APP_SQI_ID_0, conThr);

  if (txThr !=0)
  {
      PLIB_SQI_TxBufferThresholdSet(APP_SQI_ID_0, txThr);
      PLIB_SQI_TxBufferThresholdIntSet(APP_SQI_ID_0, txThr);
  }
  if (rxThr != 0)
  {
    PLIB_SQI_RxBufferThresholdSet(APP_SQI_ID_0, rxThr);
    PLIB_SQI_RxBufferThresholdIntSet(APP_SQI_ID_0, rxThr);
  }
}


// *****************************************************************************
// *****************************************************************************
// Function: Soft delay functions (local)
// *****************************************************************************
// *****************************************************************************
inline static uint32_t _APP_SQI_ReadCoreTimer()
{
  volatile uint32_t timer;

  // get the readSize reg
  asm volatile("mfc0   %0, $9" : "=r"(timer));

  return(timer);
}


inline static void _APP_SQI_StartCoreTimer(uint32_t period)
{
  /* Reset the coutner */
  volatile uint32_t loadZero = 0;

  asm volatile("mtc0   %0, $9" : "+r"(loadZero));
  asm volatile("mtc0   %0, $11" : "+r" (period));
}


inline void _APP_SQI_CoreTimer_Delay(uint32_t delayValue)
{
  while ((_APP_SQI_ReadCoreTimer() <= delayValue))
  asm("nop");
}


void APP_SQI_Flash_Initialize (void)
{
  /* Setup CONTROL, TX and RX buffer thresholds */
  _APP_SQI_BufferThrSet(1, 4, 4);

  /* Reset the SQI Flash */
  PLIB_SQI_ControlWordSet(APP_SQI_ID_0, 1, APP_SQI_CS_1, APP_SQI_LANE_SINGLE, APP_SQI_CMD_TRANSMIT, 1);   /* NOP */
  PLIB_SQI_ControlWordSet(APP_SQI_ID_0, 1, APP_SQI_CS_1, APP_SQI_LANE_SINGLE, APP_SQI_CMD_TRANSMIT, 1);   /* NOP */
  PLIB_SQI_ControlWordSet(APP_SQI_ID_0, 1, APP_SQI_CS_1, APP_SQI_LANE_SINGLE, APP_SQI_CMD_TRANSMIT, 1);   /* NOP */
  PLIB_SQI_ControlWordSet(APP_SQI_ID_0, 1, APP_SQI_CS_1, APP_SQI_LANE_SINGLE, APP_SQI_CMD_TRANSMIT, 1);   /* RSTEN */
  PLIB_SQI_ControlWordSet(APP_SQI_ID_0, 1, APP_SQI_CS_1, APP_SQI_LANE_SINGLE, APP_SQI_CMD_TRANSMIT, 1);   /* NOP */
  PLIB_SQI_ControlWordSet(APP_SQI_ID_0, 1, APP_SQI_CS_1, APP_SQI_LANE_SINGLE, APP_SQI_CMD_TRANSMIT, 1);   /* NOP */
  PLIB_SQI_ControlWordSet(APP_SQI_ID_0, 1, APP_SQI_CS_1, APP_SQI_LANE_SINGLE, APP_SQI_CMD_TRANSMIT, 1);   /* NOP */
  PLIB_SQI_ControlWordSet(APP_SQI_ID_0, 1, APP_SQI_CS_1, APP_SQI_LANE_SINGLE, APP_SQI_CMD_TRANSMIT, 1);   /* RST */
  /* Transmit reset enable */
  PLIB_SQI_TransmitData(APP_SQI_ID_0, (FLASH_RSTEN <<24) | (FLASH_NOP<< 16) |(FLASH_NOP << 8) | FLASH_NOP);

  /* Wait for 25ms for the reset cycle to be enabled */
  _APP_SQI_StartCoreTimer(0);
  _APP_SQI_CoreTimer_Delay(500000);

  /* Transmit reset */
  PLIB_SQI_TransmitData(APP_SQI_ID_0, (FLASH_NOP << 24) | (FLASH_NOP << 16) | (FLASH_NOP << 8) | FLASH_RST);

  /* Wait for 1 ms for the rest to be finished */
  _APP_SQI_StartCoreTimer(0);
  _APP_SQI_CoreTimer_Delay(100000);
}


void APP_SQI_XIP_SetXipMode (void)
{
  // Configure XIP Control 1 SFR (quad lane fast read)
  PLIB_SQI_XIPControlWord1Set(APP_SQI_ID_0,APP_DUMMY_BYTES_3,APP_ADDR_BYTES_3,
                              FLASH_FAST_READ,APP_SQI_LANE_QUAD,
                              APP_SQI_LANE_QUAD,APP_SQI_LANE_QUAD,
                              APP_SQI_LANE_QUAD,APP_SQI_LANE_QUAD);

  // Configure XIP Control 2 SFR (chip select 1)
  PLIB_SQI_XIPControlWord2Set(APP_SQI_ID_0,APP_SQI_CS_1,APP_MODE_BYTES_0,0);

  // Set to XIP mode
  PLIB_SQI_TransferModeSet(APP_SQI_ID_0, APP_SQI_XFER_MODE_XIP);
}


SQI_STATUS APP_SQI_Flash_ID_Check (void)
{
  /* Configure SQI for PIO mode */
  PLIB_SQI_TransferModeSet(APP_SQI_ID_0, APP_SQI_XFER_MODE_PIO);

  /* Setup CONTROL, TX and RX buffer thresholds */
  _APP_SQI_BufferThrSet(1, 4, 4);

  /* Enable TXTHR interrupt signal */
  PLIB_SQI_InterruptSignalEnable(APP_SQI_ID_0, APP_SQI_TXTHR);

  /* Configure to send the QJID command */
  PLIB_SQI_ControlWordSet(APP_SQI_ID_0,1,APP_SQI_CS_1,APP_SQI_LANE_SINGLE,APP_SQI_CMD_TRANSMIT,1);        /* EQIO */
  PLIB_SQI_ControlWordSet(APP_SQI_ID_0,1,APP_SQI_CS_1,APP_SQI_LANE_QUAD,APP_SQI_CMD_TRANSMIT,1);          /* NOP */
  PLIB_SQI_ControlWordSet(APP_SQI_ID_0,0,APP_SQI_CS_1,APP_SQI_LANE_QUAD,APP_SQI_CMD_TRANSMIT,2);          /* QJID & DUMMY */
  PLIB_SQI_ControlWordSet(APP_SQI_ID_0,1,APP_SQI_CS_1,APP_SQI_LANE_QUAD,APP_SQI_CMD_RECEIVE,4);           /* ID READ */

  //Write the command to the transfer buffer
  PLIB_SQI_TransmitData(APP_SQI_ID_0, (FLASH_NOP << 24) | (FLASH_QJID << 16) | (FLASH_NOP << 8) | (FLASH_EQIO));

  while (PLIB_SQI_InterruptFlagGet(APP_SQI_ID_0, APP_SQI_RXTHR) == false);

  jedecID = PLIB_SQI_ReceiveData(APP_SQI_ID_0);

  jedecID = jedecID & 0x00FFFFFF;

  if (jedecID != FLASH_JEDECID_B)
    return SQI_STATUS_FAILURE;
  else
    return SQI_STATUS_SUCCESS;
}
#endif


/*******************************************************************************
* FUNCTION:
*   EwBspSystemInit
*
* DESCRIPTION:
*   The function EwBspSystemInit initializes the system components. 
*   (CPU clock, memory, qspi, ...)
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspSystemInit( void )
{
#if EW_USE_EXTERNAL_FLASH == 1

  SQI_STATUS sqiStatus;

  APP_SQI_Initialize();
  APP_SQI_Flash_Initialize();
  while ( 1 )
  {
    sqiStatus = APP_SQI_Flash_ID_Check();
    if ( sqiStatus ==  SQI_STATUS_SUCCESS )
      break;

    EwPrint( "APP_SQI_Flash_ID_Check failed\n" );
  }

  APP_SQI_XIP_SetXipMode();

#endif
}


/*******************************************************************************
* FUNCTION:
*   EwBspSystemDone
*
* DESCRIPTION:
*   The function EwBspSystemDone terminates the system components.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspSystemDone( void )
{
}


/* msy */
